/*****************************************************************************************
 *
 *  Copyright 2006+ by Cieszak
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 ****************************************************************************************/

#include "main.h"

/* application state */
struct application *application;

#define OMSG_NAVI             (0x1f00) /* sent when Navi key is pressed (set below in "...add_5e0_id(..." line) */
#define OMSG_MENU          	(0x1f02)
#define OMSG_DRAW             (0x1f07)
#define OMSG_READ_FILE        (0x1f08)
//options
#define OMSG_SEND_VIA_SMS   	(0x1f03)
#define OMSG_ADD           	(0x1f04)
#define OMSG_EDIT          	(0x1f05)
#define OMSG_DELETE        	(0x1f06)

extern const nokstr_t str_select;
extern const nokstr_t str_msg_sending;
extern const nokstr_t str_msg_no_smsc;
extern const nokstr_t str_msg_no_network;

extern struct dialog dlg_prog;
extern struct dialog dlg_key;

// Options menu; created in _project.nrx
extern const struct menu options_menu;

/**************************************************************************************/


/* Build PDU and requests the phone to send it. Phone numbers specified
 * as parameters must be in BCD format.
 */
static void send_sms(unsigned char *smsc, unsigned char *dest, char *text)
{
   struct sms_send *sms;
   unsigned i,len;
   unsigned char *data;

   /* Get text length. */
   len=strlen(text);
   if(len>160) len=160;

   /* Allocate memory for sms_send structure (containing the PDU). */
   i=sizeof(struct sms_send)+sms_send_padding+24+len+1;
   sms=(struct sms_send *)malloc(i);
   memset(sms,0,i);

   /* Set the type. */
   sms->type=SMS_TYPE_OUTGOING;

   /* Set SMS Centre Phone Number. */
   memcpy(sms->smsc,smsc,12);

   /* Build the PDU. */
   data=sms->data+sms_send_padding; /* Skip possible padding. */
   data[0]=0x11; /* First octet */
   data[4]=len; /* TP-User-Data-Length */

   memcpy(data+5,dest,12); /* Destination's Phone Number */

   data[17]=0xA7; /* Validity Period = 24h */

   /* Encode the text in 7bits GSM Alphabet. */
   i=pack_7bits_string((char *)data+24,text,len);
   data[24+i]=0;

   /* Request the phone to send the PDU. */
   send_message(MSG_SMS_REQUEST_SEND|MSG_3P,SMS_SYS_UI,0,sms);
}

/**************************************************************************************/

/* UI messages dispatcher */
extern rexx_uint keys_dispatcher_id; /* dispatcher's call id */
rexx("keys_dispatcher_id=add_5e0_id(keys_dispatcher,add_action(locate('str_options'),x2d(1f00)))"); /* auto creation of the id */
unsigned keys_dispatcher(unsigned msg)
{
   switch(msg)
   {
      case MSG_D_INIT: // application initialization
        if(application)
         {
            free(application);
            dprintf("application wasn't freed\n");
         }
         application = malloc(sizeof(struct application));
         dprintf("application = %p\n", application);
         memset(application, 0, sizeof(struct application));
         
         if(app.fd)
         {
            tfserror(tfsclose(app.fd));
            dprintf("awaryjnie closing file %d %s\n", app.fd, tfsfdpath(app.fd));
         }
         freegfx_open_ex(0);

         /* setting fonts */
         app.app_font = own_get_font("small/bold");
         app.key_font = own_get_font("small/plain");
         app.counter_font = own_get_font("small/bold");
         app.filename_font = own_get_font("console/4");

         TFILE* file;
         if(msg_argc == 3) //from filemanager
         {
            struct fman_state *fman=(struct fman_state *)msg_argv[2];
            file = fman->files[fman->curr];
         }
         else
            file = (TFILE *)msg_argv[1];

         dprintf("Otwieram plik  %s\n", tfspath(file));

         tfserror(app.fd = tfsopen(tfspath(file), TFS_APPEND));

         send_message(OMSG_READ_FILE);
         app.state=STATE_NORMAL;

         msg=MSG_R_STAY;
         break;

      case MSG_D_QUIT: // application deinitialization

         dprintf("closing file %d\n", app.fd);
         tfserror(tfsclose(app.fd));

         app.fd = 0;

         freegfx_close();

         free(app.info);
         free(application);
         application = NULL;

         msg=MSG_R_QUIT;
         break;
         
      case OMSG_NAVI:
         // open Options menu
         send_message(MSG_OPEN_STATIC_MENU|MSG_1P,&options_menu);
         
         app.state=STATE_MENU;
         msg=MSG_NOP;
         break;
         
      case MSG_KEYPRESS_CHAR:
         //scroll
         if (app.state==STATE_NORMAL)
         {
            app.row++;
            app.row %= app.rows;

            send_message(OMSG_DRAW);
            msg=MSG_NOP;
         }
         break;
         
      case OMSG_SEND_VIA_SMS:
         if(logged_into_network)
         {
            app.state=STATE_PHONE;
            dialog_phone_open(NULL);
         }
         else
         {
            display_msg(DMI_ERROR, &str_msg_no_network);
            send_message(MSG_BACK);
         }

         msg=MSG_NOP;
         break;

      /* adding keys */
      case OMSG_ADD:
         if(app.edit_buffer)
            free(app.edit_buffer);
         app.edit_buffer = NULL;
         app.edit_buffer = malloc(DLG_BUFFER*2);
         app.edit_buffer[0] = 0;
         send_message(MSG_ALPHANUMERIC_MODE);
         own_alphanumeric_keyboard(KBT_NORMAL);
         dialog_toggle_case();
         dialog_open(app.edit_buffer, &dlg_prog);
         app.state=STATE_DLG_ADD_APP;

         msg = MSG_NOP;
         break;

      case OMSG_EDIT:
         if(app.edit_buffer)
            free(app.edit_buffer);
         app.edit_buffer = NULL;
         app.edit_buffer = malloc(DLG_BUFFER*2);
         strcpy(app.edit_buffer, app.info);
         //truncate key, leave only app name
         {
            int i;
            for(i=0; i<strlen(app.edit_buffer); i++)
            {
               if(app.edit_buffer[i] == '=')
               {
                  app.edit_buffer[i] = 0;
                  break;
               }
            }
         }
         
         send_message(MSG_ALPHANUMERIC_MODE);
         own_alphanumeric_keyboard(KBT_NORMAL);
         dialog_toggle_case();
         dialog_open(app.edit_buffer, &dlg_prog);
         app.state=STATE_DLG_EDIT_APP;

         msg = MSG_NOP;
         break;
         
      case OMSG_DELETE:
         tfserror(tfstextdeleteln(app.fd, app.current));

         send_message(OMSG_READ_FILE);
         msg = MSG_NOP;
         break;

      case MSG_KEY_DOWN:
         if(app.state==STATE_NORMAL)
         {
            app.current++;
            app.current%=app.all;
            app.row=0;
            tfserror(tfstextreadln(app.fd, app.current, app.info, app.info_length));

            send_message(OMSG_DRAW);
            msg=MSG_NOP;
         }
         break;

      case MSG_KEY_UP:
         if(app.state == STATE_NORMAL)
         {
            if (app.current == 0)
            {
               if(app.all > 0)
                  app.current = app.all-1;
            }
            else
               app.current--;
            app.row=0;

            tfserror(tfstextreadln(app.fd, app.current, app.info, app.info_length));

            send_message(OMSG_DRAW);
            msg=MSG_NOP;
         }
         break;
         
      case MSG_DIALOG_ACCEPTED:
         if(app.state==STATE_PHONE)
         {
            /* Phone number dialog was accepted. */

            /* Since phonebook could be called from phone number dialog, its upper
             * mode could be left. This results in the phonebook icon to be left on
             * the top of screen. We remove it by setting the upper mode back to menu
             * (default state).
             */
            upper_mode(UPM_MENU);

            /* Open the progress bar. */
            send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_PROGRESS,&str_msg_sending);

            /* We start the SMS sending operation by requesting SMS settings. */
            send_message(MSG_SMS_REQUEST_GET_SETTINGS|MSG_2P,SMS_SYS_UI,1);
         }
         else
         if(app.state == STATE_DLG_ADD_APP)
         {
            send_message(MSG_ALPHANUMERIC_MODE);
            own_alphanumeric_keyboard(KBT_NORMAL);
            int i = strlen(app.edit_buffer);
            app.edit_buffer[i] = '=';
            app.edit_buffer[i+1]='\0'; //none needed, maybe
            dialog_toggle_case();
            dialog_open(app.edit_buffer+i+1, &dlg_key);
            app.state=STATE_DLG_ADD_KEY;
         }
         else
         if(app.state == STATE_DLG_ADD_KEY)
         {
            tfserror(tfstextappendln(app.fd, app.edit_buffer));
            free(app.edit_buffer);
            app.edit_buffer = NULL;
            send_message(OMSG_READ_FILE);
         }
         else
         if(app.state == STATE_DLG_EDIT_APP)
         {
            send_message(MSG_ALPHANUMERIC_MODE);
            own_alphanumeric_keyboard(KBT_NORMAL);
            int i = strlen(app.edit_buffer);
            app.edit_buffer[i] = '=';
            dprintf("app after editing:'%s'\n", app.edit_buffer);
            /* find key start */
            int j;
            for(j=0; j<strlen(app.info); j++)
            {
               if(app.info[j] == '=')
               {
                  j++;
                  break;
               }
            }
            strcpy(app.edit_buffer+i+1, app.info+j);
            dialog_toggle_case();
            dialog_open(app.edit_buffer+i+1, &dlg_key);
            app.state=STATE_DLG_EDIT_KEY;
         }
         else
         if(app.state == STATE_DLG_EDIT_KEY)
         {
            dprintf("app=key for write:'%s'\n", app.edit_buffer);
            tfserror(tfstextreplaceln(app.fd, app.current, app.edit_buffer));
            free(app.edit_buffer);
            app.edit_buffer = NULL;
            send_message(OMSG_READ_FILE);
         }

         msg=MSG_NOP;
         break;
         
      case MSG_DIALOG_CANCELED:
         /* Phone number dialog sends this message instead of MSG_BACK when it is
          * cancelled. We just send the missing MSG_BACK message here.
          */
         send_message(MSG_BACK);
         break;

      case MSG_BACK:
         dprintf("MSG_BACK state=%d\n", app.state);
         if(app.state==STATE_PHONE)
         {
            dprintf("STATE_PHONE\n");
            /* If the progress bar was opened, close it. */
            send_message(MSG_CLOSE_DISPLAY_MESSAGE_DMI|MSG_1P,DMI_PROGRESS);

            app.state=STATE_NORMAL;
            send_message(OMSG_DRAW);

            msg=MSG_NOP; /* This prevents application from quitting. */
         }
         else
         if (app.state == STATE_MENU ||
             app.state == STATE_DLG_ADD_APP ||
             app.state == STATE_DLG_EDIT_APP ||
             app.state == STATE_DLG_ADD_KEY ||
             app.state == STATE_DLG_EDIT_KEY)
         {
            dprintf("STATE_MENU\n");
            app.state=STATE_NORMAL;
            send_message(OMSG_DRAW);
            msg=MSG_NOP; /* This prevents application from quitting. */
         }
         else
         
            dprintf("OTHER STATE\n");
         break;

      case MSG_SMS_REPLY_OK_GET_SETTINGS:
         {
            /* Ok, we've received the SMS settings. So we have access to the SMS
             * Centre Number, we also have the text and destination phone number
             * from the dialogs. So, we can actualy send the SMS message here.
             */
            struct sms_settings *sett=(struct sms_settings *)msg_argv[0];
            unsigned char dest[12];

            /* First, we encode the destination phone number in BCD code. */
            encode_bcd_phone(dest,dialog_phone_get_phone());

            /* And we call our function that will build PDU and send the message. */
//            char text[strlen(app.info)];
//            tfserror(tfstextreadln(app.fd, app.current, text, 0));
//            send_sms(sett->smsc,dest,text);
            send_sms(sett->smsc, dest, app.info);
         }
         break;

      case MSG_SMS_REPLY_FAIL_GET_SETTINGS:
         /* Requested SMS settings set does not exist. This shouldn't happend
          * because we've requested the default set (1). We just go back to
          * text dialog (going back will close the progress bar too).
          */
         display_msg(DMI_ERROR, &str_msg_no_smsc);
         msg=MSG_NOP;
         break;

      case MSG_SMS_REPLY_OK_SEND:
//         display_msg(DMI_ERROR, (const nokstr_t *)"Wyslano");
         /* The SMS message was sent successfully. We're not discarding this message
          * so the system will display "Message sent" message. We just quit the
          * application, the progress bar will be closed in MSG_D_QUIT.
          */
         send_message(MSG_BACK);
         break;

      case MSG_SMS_REPLY_FAIL_SEND:
         /* The SMS message couldn't be sent. We're not discarding this message so
          * the system will display "Message send failure" message. We just go back
          * to text dialog (going back will close the progress bar too).
          */
         send_message(MSG_BACK);
         break;

      case OMSG_READ_FILE:
         dprintf("READ_FILE\n");
         app.state = STATE_NORMAL;
         tfserror(app.all = tfstextcount(app.fd, &app.info_length));
         app.info_length += 1; //one byte for NUL character
         app.current = 0;
         app.row = 0;
         /* allocating memory for draw buffer */
         if(app.info)
            free(app.info);
         app.info = malloc(app.info_length);
         dprintf("app.info=%p\n", app.info);

         dprintf("READ_FILE all=%d info_length=%d\n", app.all, app.info_length);

         tfserror(tfstextreadln(app.fd, app.current, app.info, app.info_length));

         send_message(OMSG_DRAW);
         msg = MSG_NOP;
         break;

      case OMSG_DRAW:
      {
         freegfx_begin(true); // begin screen drawing session (with screen clearing)
         int row=0;

         char counter[8];
         counter[0] = 0;
         sprintf(counter, " %d/%d", app.all?app.current+1:app.current, app.all);

         char *filename=tfsfdpath(app.fd);
         //only file name
         int i;
         for(i = strlen(filename); i>0; i--)
            if(filename[i] == '/')
               break;

         own_print_string((const nokstr_t *)(filename + i + 1), 0, 5, app.filename_font, DM_SET);
         own_print_string((const nokstr_t *)counter,screen_info.width+4-6*strlen(counter), 6, app.counter_font, DM_SET);

         int y=14;
         int pos=0;
         int len = strlen(app.info);
         char *buff = malloc(len+1);
         dprintf("buff=%p\n", buff);
         struct font *f=app.app_font;
         dprintf("redraw_screen, app.info=%s strlen(app.info)=%d\n", app.info, len);

         while(pos < len)
         {
            int buffpos=0;
            buff[buffpos]='\0';
            do
            {
               buff[buffpos]=app.info[pos];
               buff[buffpos+1]='\0';
               if(own_get_string_width((const nokstr_t *)buff, f)>screen_info.width)
               //longer than screen width
               {
                  buff[buffpos]='\0'; //cut last char
                  break;
               }
               buffpos++;
               pos++;
            }
            while((app.info[pos]!='=') && (app.info[pos]!='\0'));

            if(app.row<=row)
            {
               own_print_string((const nokstr_t *)buff, 0, y, f, DM_SET);
               y+=8;
            }
            row++;
            if(app.info[pos]=='=')
            {
               f = app.key_font;
               pos++;
            }
         }
         free(buff);
         app.rows=row-3;
         if(app.rows < 0)
            app.rows=0;

         freegfx_end(false); // end screen drawing session
      }
         break;

   }
   return msg;
}

/**************************************************************************************/

