/*****************************************************************************************
 *
 *  Copyright 2006+ by Cieszak
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 ****************************************************************************************/

#include <nokix/all.h>
//#define DEBUG
#include "tfstext.h"



#define OMSG_SELECT_FILE     	(0x1f00)
#define OMSG_NEW_FILE       	(0x1f01)
#define OMSG_CREATE_NEW_FILE 	(0x1f02)
#define OMSG_SEARCH 	         (0x1f03)
#define OMSG_START_SEARCH 	   (0x1f04)
#define OMSG_SEARCH_BAR 	   (0x1f05)

#define STATE_MENU               0
#define STATE_NEW_FILE_DIALOG    1

struct menu_application
{
   struct dialog *dlg;
   TFILE *fp;
   char *name;
   char *path;
   char *cwd;
   unsigned char state;
};

struct menu_application *menu_application;

/* global_buffer is an array of 40 bytes in RAM which you can use for your
   own purposes as long as your application stays active; when it is deactivated
   the buffer may be overwritten by another one;
   here we define 'app' symbol to access these 40 bytes as a structure */
#define menu_app (*(struct menu_application *)global_buffer)

#define tmpfilename "searchresults.key"

extern const nokstr_t str_msg_no_files;
extern const nokstr_t str_msg_empty_dialog;
extern const nokstr_t str_msg_created;
extern const nokstr_t str_msg_notfound;
extern const nokstr_t str_msg_searching;
extern nokstr_t str_search;
extern nokstr_t str_new_file;

extern struct dialog dlg_prog;

extern rexx_uint max_len; /* max length of dialog buffer */


extern rexx_uint keys_dispatcher_id;
extern rexx_uint menu_dispatcher_id; /* dispatcher's call id */
rexx("menu_dispatcher_id=add_5e0_id(menu_dispatcher)"); /* auto creation of the id */


/* form pbar.c  - modified*/
void pbar_open(const nokstr_t *str)
{
   dprintf("pbar_open()\n");

   keyboard_enable(false);
   dprintf("  opening progress bar\n");
   display_msg(DMI_PROGRESS,str);
   update_screen();
}

void pbar_close(void)
{
   dprintf("pbar_close()\n");
   keyboard_enable(true);
   dprintf("  closing progress bar\n");
   close_display_msg_dmi(DMI_PROGRESS);
   update_screen();
}









static unsigned create_files_menu(void)
//create menu with files with key extension
{
   struct menu_item item;
   TFILE **tab=NULL;
   TFILE *fp=NULL;
   char ext[TFSNAMESIZE+1];
   unsigned c=0, i;

   menu_delete(1); /* delete menu 1 if it exists */
   menu_create(1,MDM_LIST|MDM_SUB,MCM_MARK_ID,1,&str_select);

   /* init our item with defaults */
   memcpy(&item,&menu_item_empty,sizeof(struct menu_item));

   /* set working dir */
   menu_app.cwd = realloc(menu_app.cwd, 2);
   strcpy(menu_app.cwd, "/");

   //if keys directory exists, it will be used
   TFILE *keysdir;
   char keyspath[]="/keys";
   if(tfsvalid(keysdir=tfsstat(keyspath)))
      if(TFS_DIRECTORY(keysdir))
      {
         menu_app.cwd = realloc(menu_app.cwd, strlen(keyspath)+1);
         strcpy(menu_app.cwd, keyspath);
      }
         

   while((fp=tfsnext(fp, menu_app.cwd)))
   {
      if (TFS_DIRECTORY(fp)) continue; //directory
      if (tfsgetext(ext,TFS_NAME(fp))!=TFS_OKAY) continue; //file hasn't extension
      if (strcmp(ext,".key")) continue;

      tab=(TFILE **)realloc(tab,(c+1)*sizeof(TFILE *));
      for(i=0;i<c;i++)
         if(aord_stricmp(TFS_NAME(fp),TFS_NAME(tab[i]))<0)
         {
            memmove(tab+i+1,tab+i,(c-i)*sizeof(TFILE *));
            break;
         }

      tab[i]=fp;
      c++;
   }

   for(i=0;i<c;i++)
   {
      item.title=(nokstr_t *)TFS_NAME(tab[i]);
      item.msg=OMSG_SELECT_FILE;
      item.arg=(unsigned)tab[i];
      item.action=action_ok_back;
      menu_add_item(1,0,&item); /* add item at end */
   }
   
   /* new file */
   item.title = &str_new_file;
   item.msg=OMSG_NEW_FILE;
   item.arg=0;
   item.action=action_ok_back;
   menu_add_item(1,0,&item); /* add item at end */

   /* search */
   item.title=&str_search;
   item.msg=OMSG_SEARCH;
   item.arg=0;
   item.action=action_ok_back;
   menu_add_item(1,1,&item); /* add item at end */

   free(tab);
   return c;
}


ULONG menu_dispatcher(ULONG msg)
{
   switch(msg)
   {
      case MSG_D_INIT: // application initialization
      
      
         create_files_menu();
         if(menu_application)
         {
            free(menu_application);
         }
         menu_application = malloc(sizeof(struct menu_application));
         memset(menu_application, 0, sizeof(struct menu_application));
         menu_open(1,1);

         msg=MSG_R_STAY;
         break;

      case MSG_D_QUIT: // application deinitialization
         freegfx_close();
         menu_delete(1);
         free(menu_app.name);
         menu_app.name=NULL;
         free(menu_application);
         menu_application = NULL;
         msg=MSG_R_QUIT;
         break;
         
      case MSG_D_QUIT_NOW: // called when we get back from some other dispatcher
         create_files_menu();
         menu_reopen();
         msg=MSG_R_STAY;
         break;

      case OMSG_SELECT_FILE:
         //run keys dispatcher with selected file pointer
         send_message(MSG_START_HANDLER|MSG_2P,keys_dispatcher_id, msg_argv[0]);
         msg=MSG_NOP;
         break;
         
      case OMSG_NEW_FILE:
         menu_app.name=malloc(TFSNAMESIZE+1);
         menu_app.dlg=malloc(sizeof(struct dialog));
         memcpy(menu_app.dlg,&dialog_filename,sizeof(struct dialog));
         menu_app.dlg->max_length=TFSNAMESIZE-4; // for extension

         menu_app.name[0]=0;
         dialog_open(menu_app.name,menu_app.dlg);
         dialog_toggle_case();

         msg=MSG_NOP;
         break;

      case MSG_DIALOG_ACCEPTED:
      {
         int err;
         if(menu_app.name[0]==0)
         {
            // empty name dialog was accepted
            dialog_open(menu_app.name, menu_app.dlg);
            dialog_toggle_case();
            send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_INFORMATION,&str_msg_empty_dialog);
         }
         else
         {
            // check if the name is valid (does not contain illegal chars)
            err=tfschkname(menu_app.name);
            if(err==TFS_OKAY)
            {
               char *path;
               // build the full path
               path = malloc(strlen(menu_app.cwd)+strlen(menu_app.name)+4+1);
               strcpy(path, menu_app.cwd);
               if (path[strlen(path)-1] != '/')
                   strcat(path, "/");
               strcat(path, menu_app.name);
               strcat(path, ".key");

               // open overwrite asking dialog if needed
               if((err = fman_ask_overwrite(path, MSG_NOP))==TFS_OKAY)
               {
                  // try to write the file
                  if((err = tfsadd(path, 0, NULL, 0))==TFS_OKAY)
                  {
                     send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_DONE,&str_msg_created);
                     send_message(MSG_D_QUIT_NOW); //recreate files menu
                     free(menu_app.name);
                     menu_app.name=NULL;
                     //TODO: start keys dispatcher with file "path"
                  }
               }
               free(path);
            }

            if(err!=TFS_OKAY)
            {
               // error, reopen the name dialog and display the error message
               dialog_open(menu_app.name,menu_app.dlg);
               dialog_toggle_case();
               tfserror(err);
            }
         }
         break;
      }

      /* searching */
      case OMSG_SEARCH:
         send_message(MSG_ALPHANUMERIC_MODE);
         own_alphanumeric_keyboard(KBT_NORMAL);
         dialog_toggle_case();
         menu_app.name = malloc(max_len);
         dprintf("pokazaem dialog\n");
         dialog_open_msg(menu_app.name, &dlg_prog, OMSG_START_SEARCH);

         msg=MSG_NOP;
         break;

      case OMSG_START_SEARCH:
      {
         dprintf("rozpoczynam szukanie\n");
         pbar_open(&str_msg_searching);

         /* create tmp file */
         int fd;
         char *path;
         // build the full path
         path = malloc(strlen(menu_app.cwd)+strlen(tmpfilename)+1);
         strcpy(path, menu_app.cwd);
         if (path[strlen(path)-1] != '/')
            strcat(path, "/");
         strcat(path, tmpfilename);

         dprintf("-------------dane zapisywane do pliku: %s\n", path);
         /* remove file if exists, using TFS_CREATERM seems not wor */
         if (tfsstat(path)!=NULL)
            tfsunlink(path);
         tfserror(fd = tfsopen(path, TFS_CREATERM|TFS_A_HIDDEN));
         dprintf("fd = %d\n", fd);

         /* searching files */
         TFILE *fp=NULL;
         char ext[TFSNAMESIZE+1];

         int found = 0;
         
         while((fp=tfsnext(fp, menu_app.cwd)))
         {
            if (TFS_DIRECTORY(fp)) continue; //directory
            if (tfsgetext(ext,TFS_NAME(fp))!=TFS_OKAY) continue; //file hasn't extension
            if (strcmp(ext,".key")) continue;

            dprintf("szukam w pliku %s, tekstu %s\n", tfspath(fp), menu_app.name);
            int curfd;
            tfserror(curfd=tfsopen(tfspath(fp), TFS_RDONLY));

            int maxlinelength;
            tfserror(tfstextcount(curfd, &maxlinelength));
            dprintf("maxlinelength = %d\n", maxlinelength);
            char line[maxlinelength+1];
            tfserror(tfsseek(curfd, 0, TFS_BEGIN));
            while(!tfseof(curfd))
            {
               tfserror(tfstextreadnextln(curfd, line, sizeof(line)));
               dprintf("linia %s\n", line);

               if (strstr(line, menu_app.name) != NULL)
               {
                  found = 1;
                  tfserror(tfstextwriteln(fd, line));
               }
            }
            tfserror(tfsclose(curfd));
         }
         tfserror(tfsclose(fd));
         free(menu_app.name);
         /* If the progress bar is still opened, we close it. */
         dprintf("---chowam progressbar\n");
         pbar_close();
         if(found)
         {
            dprintf("---uruchamiam keys_dispatcher_id\n");
            //run keys dispatcher with selected file pointer
            send_message(MSG_START_HANDLER|MSG_2P,keys_dispatcher_id, tfsstat(path));
            msg=MSG_NOP;
         }
         else
         {
            dprintf("---chowam progressbar\n");
            //not found
            send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_ERROR,&str_msg_notfound);
            msg=MSG_REQUEST_QUIT;
         }
         free(path);
      }
         msg=MSG_NOP;
         break;
   }
   return msg;
}


