/*****************************************************************************************
 *
 *  Copyright 2006+ by Cieszak
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 ****************************************************************************************
 *
 *  Version    Date          Author             Log
 *  --------------------------------------------------------------------------------------
 *  1.0        2006.07.19    Cieszak            Inital Release (tfstextcount,
 *                                              tfstextreadln, tfstextappendln)
 *
 *  1.1        2006.08.03    Cieszak            Add rest of functions
 *                                              (tfstextinsertln, tfstextreplaceln,
 *                                              tfstextdeleteln, tfstextmode)
 *
 *  1.2        2006.08.14    Cieszak            Full working version
 *
 *  1.3        2006.08.28    Cieszak            Rewrited and optimized all of the code 
 *												(added tfstextgotonextln, tfstextgotoln, 
 *												tfstextreadnextln, tfstextwriteeol, 
 *												tfstextwriteln functions)
 *
 *  1.4        2006.09.22    Cieszak            Another optimizations
 *
 *  1.5        2006.12.04    Cieszak            Corrected documentation
 *
 ****************************************************************************************/

#include "tfstext.h"

/******************************************************************************/

int tfstextgotonextln(int fd)
{
   dprintf("tfstextgotonextln(%d)\n", fd);
   long pos = tfstell(fd);
   if(pos < 0)
      return pos;
      
   char *buffer = malloc(MAX_MEM_BUFFER);
   unsigned char end=0;
   int result, bytes_counter = 0;
   char r=0;    //if previous char was \r
   int i;

   while((result = tfsread(fd, buffer, MAX_MEM_BUFFER))>0)
   {
      for(i = 0; i < result; i++)
      {
         pos++;
         if((buffer[i]!='\n') && (buffer[i]!='\r'))
         {
            if(buffer[i] == 0)
            {
               free(buffer);
               return TFSERR_NOTTEXTFILE;
            }

            if(r) // \r, other char
            {
               end=1;
               break;
            }
            bytes_counter++;
         }
         else if(buffer[i]=='\n') // \n; \r:\n
         {
            end=1;
            break;
         }
         else if(buffer[i]!='\r')
         {
            r=1;
         }
      }
      if(end)
         break;
   }
   free(buffer);
   if(result < 0)
      return result;

   result = tfsseek(fd, pos, TFS_BEGIN);
   if(result < 0)
      return result;
      
   dprintf("### pos = %d, size =%d\n", tfstell(fd), tfsfdsize(fd));
      
   return bytes_counter;
}

int tfstextcount(int fd, int *maxlnlen)
{
   dprintf("tfstextcount(%d, %p)\n", fd, maxlnlen);
   int count = 0, result, max_line_length;
   long pos;

   /* save current position */
   pos = tfstell(fd);
   if(pos < 0)
      return pos;
      
   /* seek to begin */
   result = tfsseek(fd, 0, TFS_BEGIN);
   if(result < 0)
      return result;

   /* count lines and set maxlnlen (if maxlnlen!=NULL) to max length of line in this
      file */
   while((max_line_length = tfstextgotonextln(fd)) >= 0)
   {
      count++;
      if(maxlnlen)
         if(*maxlnlen < max_line_length)
            *maxlnlen = max_line_length;
   }

   /* restore file position */
   result = tfsseek(fd, pos, TFS_BEGIN);
   if(result < 0)
      return result;
      
   return count;
}

int tfstextreadnextln(int fd, char *text, const int len)
{
   long pos = tfstell(fd);
   unsigned char end = 0;
   int result;
   int bufi = 0;  //text buffer index (and written bytes counter)
   char r = 0;    //if previous char was \r
   int i;
   char *buffer = malloc(MAX_MEM_BUFFER);

   while((result = tfsread(fd, buffer, MAX_MEM_BUFFER))>0)
   {
      for(i = 0; i < result; i++)
      {
         if((buffer[i]!='\n') && (buffer[i]!='\r'))
         {
            if(r) // \r:other char
            {
               end=1;
               break;
            }

            if (bufi<len-1)
            {
               text[bufi++]=buffer[i];
            }
            pos++;
         }
         else if(buffer[i]=='\n') // \n; \r:\n
         {
            end=1;
            pos++;
            break;
         }
         else if(buffer[i]!='\r')
         {
            r=1;
            pos++;
         }
      }
      if(end)
         break;
   }
   free(buffer);

   tfsseek(fd, pos, TFS_BEGIN);
   text[bufi]=0;

   return bufi?bufi:result;
}

int tfstextreadln(int fd, const int num, char *text, int len)
{
   int result;

   result = tfstextgotoln(fd, num);
   if(result < 0)
      return result;
   
   return tfstextreadnextln(fd, text, len);
}

int tfstextwriteeol(const int fd, const int mode)
{
   dprintf("tfstextwriteeol(%d, %d)\n", fd, mode);
   switch (mode)
   {
      case WIN_MODE:
         return tfswrite(fd, "\r\n", 2);
      case LIN_MODE:
         return tfswrite(fd, "\n", 1);
      case MAC_MODE:
         return tfswrite(fd, "\r", 1);
   }
   return TFSERR_BADARG;
}

int tfstextwriteln(int fd, char *text)
{
   dprintf("tfstextwriteln(%d, %s)\n", fd, text);
   int bytecount=0, result;
   int mode = tfstextmode(fd); //check file mode

   /* write line */
   result = tfswrite(fd, text, strlen(text));
   if (result < 0)
      return result;
   bytecount += result;

   /* write EOL at end of file */
   result = tfstextwriteeol(fd, mode);
   if (result < 0)
      return result;
   bytecount += result;
   
   return bytecount;
}

int tfstextappendln(int fd, char *text)
{
   dprintf("tfstextappendln(%d, %s)\n", fd, text);
   int mode = tfstextmode(fd); //check file mode
   dprintf("  return %d\n", mode);
   int bytecount=0, result;

   if(tfsfdsize(fd)>0)
   {
      //checking last character
      result = tfsseek(fd, 1, TFS_END);
      dprintf("  tfsseek return %d\n", result);
      if(result < 0)
         return result;
      char buffer;
      result = tfsread(fd, &buffer, 1);
      dprintf("  tfsread return %d\n", result);
      if(result < 0)
         return result;

      //if file isn't ended by EOL, write it
      if(buffer!='\n' && buffer!='\r') {
         result = tfstextwriteeol(fd, mode);
         dprintf("  return %d\n", result);
         if(result < 0)
            return result;
         bytecount += result;
      }
   }
   result = tfstextwriteln(fd, text);
   dprintf("  return %d\n", result);
   if(result < 0)
      return result;
   bytecount += result;

   return bytecount;
}

int tfstextinsertln(int fd, const int num, char *text)
{
   dprintf("tfstextinsertln(%d, %d, %s)\n", fd, num, text);
   int bytecount=0, result;

   result = tfstextgotoln(fd, num);
   if(result < 0)
      return result;

   long len=tfsfdsize(fd)-tfstell(fd)+1;
   char *buffer = malloc(len);
   len = tfsread(fd, buffer, len);
   if(len < 0)
      { free(buffer); return len; }

   result = tfstextgotoln(fd, num);
   if(result < 0)
      { free(buffer); return result; }

   result = tfstextwriteln(fd, text);
   if(result < 0)
      { free(buffer); return result; }
   bytecount += result;

   result = tfswrite(fd, buffer, len);
   if(result < 0)
      { free(buffer); return result; }
   bytecount += result;

   free(buffer);
   return bytecount;
}

int tfstextreplaceln(int fd, const int num, char *text)
{
   dprintf("tfstextreplaceln(%d, %d, %s)\n", fd, num, text);
   int bytecount=0, result;

   result = tfstextgotoln(fd, num+1);
   if(result < 0)
      return result;

   long len = tfsfdsize(fd)-tfstell(fd)+1;
   char *buffer = malloc(len);
   len = tfsread(fd, buffer, len);
   if(len < 0)
      { free(buffer); return len; }

   result = tfstextgotoln(fd, num);
   if(result < 0)
      { free(buffer); return result; }

   result = tfstextwriteln(fd, text);
   if(result < 0)
      { free(buffer); return result; }
   bytecount += result;

   /* cut useless part of file */
   if(tfstell(fd) + len < tfsfdsize(fd))
   {
      result = tfstruncate(fd, tfstell(fd)+len);
      if(result < 0)
         { free(buffer); return result; }
   }
   /* write rest of file */
   result = tfswrite(fd, buffer, len);
   if(result < 0)
      { free(buffer); return result; }
   bytecount += result;

   free(buffer);
   return bytecount;
}

int tfstextdeleteln(int fd, int num)
{
   dprintf("tfstextdeleteln(%d, %d)\n", fd, num);
   int result;
   result = tfstextgotoln(fd, num+1);
   if(result < 0)
      return result;

   if(tfsfdsize(fd) == tfstell(fd)) //removing last line
   {
      tfstextgotoln(fd, num);
      result = tfstruncate(fd, tfstell(fd));
   }
   else
   {

      long len = tfsfdsize(fd)-tfstell(fd)+1;
      char *buffer = malloc(len);
      len = tfsread(fd, buffer, len);
      if(len < 0)
         { free(buffer); return len; }

      result = tfstextgotoln(fd, num);
      if(result < 0)
         { free(buffer); return result; }

      /* cut useless part of file */
      if(tfstell(fd) + len < tfsfdsize(fd))
      {
         result = tfstruncate(fd, tfstell(fd)+len);
         if(result < 0)
            { free(buffer); return result; }
      }
      /* write rest of file */
      result = tfswrite(fd, buffer, len);
      if(result<0)
         { free(buffer); return result; }

      free(buffer);
   }
   return result;
}

int tfstextmode(const int fd)
{
   dprintf("tfstextmode(%d)\n", fd);
   long pos = tfstell(fd);
   
   int result = tfsseek(fd, 0, TFS_BEGIN);
   if (result < 0)
      return result;

   int i, mode = WIN_MODE, found = 0;
   char *buffer = malloc(MAX_MEM_BUFFER);

   while(found == 0 && (result = tfsread(fd, buffer, MAX_MEM_BUFFER))!=TFSERR_EOF)
   {
      if (result<0)
         return result;
      for(i = 0; i < result; i++)
      {
         if(buffer[i]=='\r')
         {
            if(buffer[i+1]=='\n')
            {
               mode=WIN_MODE;
               found=1;
               break;
            }
            else
            {
               mode=MAC_MODE;
               found=1;
               break;
            }
         }
         else
         if(buffer[i]=='\n')
         {
            mode=LIN_MODE;
               found=1;
            break;
         }
      }
   }

   //restoring position
   result = tfsseek(fd, pos, TFS_BEGIN);
   if (result < 0)
      return result;
   free(buffer);
   return mode;
}

int tfstextgotoln(int fd, int num)
{
   dprintf("tfstextgotoln(%d, %d)\n", fd, num);
   int i, result;
   result = tfsseek(fd, 0, TFS_BEGIN);
   dprintf("  tfstextgotoln(%d, %d) after seek result =%d\n", fd, num, result);
   if(result <0)
      return result;
   
   for(i = 0; i < num; i++)
   {
      dprintf("  tfstextgotoln(%d, %d) gotonextln\n", fd, num);
      if((result = tfstextgotonextln(fd)) < 0)
      {
         dprintf("  tfstextgotoln/for/gotonextln result = %d\n", result);
         return result;
      }
   }
      
   if(num > i)
   {
      dprintf("  Za mao linii w pliku\n");
      return TFSERR_EOF;
   }

   return TFS_OKAY;
}
